/**
 * Ein Objekt der Klasse Recheneinheit stellt grundlegende Rechenoperationen zur Verfuegung. 
 * Unter anderem können Verkettungen von Grundrechenoperationen berechnen werden. Dazu 
 * speichert das Objekt einen Zahlenwert als Zwischenergebnis und einen noch nicht ausgewerteten 
 * Grundrechenoperator. Sobald der zweite Operand für den gespeicherten Grundrechenoperator 
 * uebergeben wird, erfolgt eine Auswertung des gespeicherten Operators, dessen Resultat als 
 * neues Zwischenergebnis gespeichert wird. 
 */
public class Recheneinheit {
  private double zahlspeicher; //Speichert den Zahlenwert des aktuellen Zwischenergebnisses
  private String operatorspeicher; //Speichert den noch nicht ausgewerteten Grundrechenoperator

  /**
   * Initialisiert ein neues Objekt der Klasse Recheneinheit. Im Speicher 
   * ist eine 0 und es gibt keinen unausgewerteten Operator.
   */
  public Recheneinheit() {
    zuruecksetzen();
  }

  /**
   * Setzt das Objekt in den Zustand bei der Initialisierung zurueck, d. h. im Speicher
   * ist eine 0 und es gibt keinen unausgewerteten Operator.
   */
  public void zuruecksetzen(){
    zahlspeicher = 0;
    operatorspeicher = "";  
  }           

  /**
   * Liefert den aktuell gespeicherten Zahlenwert zurueck, d. h. das 
   * Zwischenergebnis der bis jetzt durchgefuehrten Grundrechenoperationen. 
   */
  public double gibZahlspeicher() {
    return zahlspeicher;
  }
  
  /**
   * Speichert den Grundrechenoperator pNaechsterOperator zur spaeteren Anwendung.
   */
  public void speichereGrundrechenoperator(String pNaechsterOperator) {
    operatorspeicher = pNaechsterOperator;
  }

  /**
   * Die Methode wendet den gespeicherten Grundrechenoperator auf den im Objekt gespeicherten 
   * Zahlenwert als ersten Operanden und den in pZahl übergebenen Wert als zweiten Operanden an.  
   * Das Ergebnis wird als neuer Zahlenwert gespeichert.
   * 
   * Gibt es keinen gespeicherten Grundrechenoperator, wird pZahl als neuer Zahlenwert gespeichert. 
   */
  public void verarbeiteGrundrechenoperator(double pZahl) {
    if (operatorspeicher.equals("")) {
      zahlspeicher = pZahl;
    } else {
      zahlspeicher = ermittleRechenergebnis(zahlspeicher, operatorspeicher, pZahl);
      operatorspeicher = "";
    }    
  }
  
  /**
   * Liefert die Quadratwurzel von pZahl zurueck.
   */
  public double liefereQuadratwurzel(double pZahl) {
    return Math.sqrt(pZahl);
  }

  /**
   * Liefert ein Hundertstel von pZahl zurueck.
   */
  public double liefereProzentwert(double pZahl) {
    return pZahl / 100;
  }

  /**
   * Liefert die Gegenzahl von pZahl zurueck.
   */
  public double liefereGegenzahl(double pZahl) {
    return -pZahl;
  }

  /**
   * Hilfsmethode: Ermittelt die Auswertung des Operators pOperator bei Anwendung auf pZahl1 und pZahl2 
   * und liefert sie als Rueckgabewert.
   */
  private double ermittleRechenergebnis(double pZahl1, String pOperator, double pZahl2) {
    double ergebnis = 0;
    if (pOperator.equals("+")) {
        ergebnis = pZahl1 + pZahl2;
    } else if(pOperator.equals("-")) {
        ergebnis = pZahl1 - pZahl2;
    } else if (pOperator.equals("*")) {
        ergebnis = pZahl1 * pZahl2;
    } else if (pOperator.equals("/")) {
        ergebnis = pZahl1 / pZahl2; 
    }
    return ergebnis;
  }

} 
