import GLOOP.*;
public class TaschenrechnerGUI {
  //Eingabeobjekte
  private GLMaus maus; 
  private GLTastatur tastatur;

  //Anzeige fuer die Zahlen (und Gehaeuse fuer die Optik)
  private Anzeige anzeige;
  private GLQuader gehaeuse;

  //Tastenobjekte
  private Taste acTaste;
  private Taste wurzelTaste;
  private Taste prozentTaste;
  private Taste vorzeichenTaste;
  private Taste plusTaste;
  private Taste minusTaste;
  private Taste malTaste;
  private Taste geteiltTaste;
  private Taste gleichTaste;
  private Taste kommaTaste;
  private Taste[] zahlenTaste;

  //Recheneinheit und Hilfsvariablen zum Aufbauen der Zahlen aus Einzelzeichen
  private Recheneinheit recheneinheit;
  private boolean neueZahl;
  private boolean komma;

  public TaschenrechnerGUI() {   
    maus = new GLMaus();
    tastatur = new GLTastatur();

    anzeige = new Anzeige();
    gehaeuse = new GLQuader(0, 0, 0, 300, 475, 1);
    gehaeuse.setzeFarbe(0,0,255);//Gehaeuse wird grau dargestellt
    acTaste = new Taste(0, 0, "AC", 255, 0, 0);
    wurzelTaste = new Taste(1, 0, "SQRT", 0, 200, 0);
    prozentTaste = new Taste(2, 0, "%", 0, 200, 0);
    vorzeichenTaste = new Taste(2, 4, "+/-", 0, 200, 0);
    plusTaste = new Taste(3, 0, "+", 0, 200, 0);
    minusTaste = new Taste(3, 1, "-", 0, 200, 0);
    malTaste = new Taste(3, 2, "*", 0, 200, 0);
    geteiltTaste = new Taste(3, 3, "/", 0, 200, 0);
    gleichTaste = new Taste(3, 4, "=", 0, 200, 0);
    kommaTaste = new Taste(1, 4, ".", 255, 15, 0);

    zahlenTaste = new Taste[10];
    for (int i = 1; i < 10; i++){
      zahlenTaste[i]=new Taste((i-1) % 3, (i-1) / 3 + 1, ""+i, 255, 15, 0);
    }
    zahlenTaste[0]= new Taste(0, 4, "" + 0, 255, 15, 0);

    recheneinheit = new Recheneinheit();
    neueZahl = true;
    komma = false;
  }

  private void setzeAnzeige(String pNeueAnzeige){
    int laenge = pNeueAnzeige.length();
    if (laenge > 14 && pNeueAnzeige.indexOf(".") >= 0) {
      int posE = pNeueAnzeige.indexOf("E");
      if (posE > 0) {
        String temp = pNeueAnzeige.substring(0, Math.min(posE-1, 10));
        pNeueAnzeige = temp + pNeueAnzeige.substring(posE);
      } else {
        pNeueAnzeige = pNeueAnzeige.substring(0, 14);
      }
    }
    anzeige.setzeText(pNeueAnzeige);
  }

  private String gibAnzeige(){
    return anzeige.gibText();
  }

  private double gibAnzeigeAlsZahl(){
    return Double.parseDouble(anzeige.gibText());
  }

  private Taste gibGeklickteTaste(int pX, int pY) {
    GLObjekt objekt = Sys.gibObjekt(pX, pY);

    int i = 0;
    Taste taste = null;
    while (taste == null && i < 10) {
      if (objekt == zahlenTaste[i].gibTafel()) {
        taste = zahlenTaste[i];
      } else {
        i = i + 1;
      }
    }
    if (taste == null) {
        if (objekt == acTaste.gibTafel()) {
            taste = acTaste;
        } else if (objekt == wurzelTaste.gibTafel()) {
            taste = wurzelTaste;
        } else if (objekt == prozentTaste.gibTafel()) { 
            taste = prozentTaste;
        } else if (objekt == vorzeichenTaste.gibTafel()) {
            taste = vorzeichenTaste;
        } else if (objekt == plusTaste.gibTafel()) {
            taste = plusTaste;
        } else if (objekt == minusTaste.gibTafel()) {
            taste = minusTaste;
        } else if (objekt == malTaste.gibTafel()) {
            taste = malTaste;
        } else if (objekt == geteiltTaste.gibTafel()) {
            taste = geteiltTaste;
        } else if (objekt == gleichTaste.gibTafel()) {
            taste = gleichTaste;
        } else if (objekt == kommaTaste.gibTafel()) {
            taste = kommaTaste;
        }    
    }
    return taste;
  }

  public void fuehreAus(){
    while (!tastatur.esc()){
      if (maus.linksklick()){
        //Die Taste, auf der die Maus gedrueckt wurde, wird ermittelt.
        Taste taste = gibGeklickteTaste(maus.gibX(), maus.gibY());           
        if (taste != null) { 
          //Die Taste wird für 100 Millisekunden grau.
          taste.setzeFarbeGedrueckt();
          Sys.warte(100);
          taste.setzeFarbeNormal();  
          String aufschrift = taste.gibAufschrift();
          switch (aufschrift) {            
            case "+": 
            case "-": 
            case "*": 
            case "/": aktionGrundrechenartTaste(aufschrift); break;
            case "=": aktionGleichTaste(); break;
            case ".": aktionKommaTaste(); break;
            case "+/-":aktionVorzeichenTaste(); break;
            case "%": aktionProzentTaste(); break;
            case "SQRT": aktionWurzelTaste(); break;
            case "AC": aktionAllClearTaste(); break;
            default : aktionZahlTaste(aufschrift);
          }
        }
      }
      Sys.warte();
    }
    Sys.beenden();
  }

  // -------------- START: Ereignismethoden -------------- 
  private void aktionZahlTaste(String pTastenzeichen) {
    String anzeigeText = gibAnzeige();
    if (anzeigeText.equals("0") || neueZahl){
      anzeigeText = pTastenzeichen;
      neueZahl = false;
    } else if (anzeigeText.length() < 14) {
      anzeigeText = anzeigeText + pTastenzeichen;
    }
    setzeAnzeige(anzeigeText);
  }

  private void aktionKommaTaste() {
    String anzeigeText = gibAnzeige();
    if (neueZahl) {
      setzeAnzeige("0.");
      neueZahl = false;
      komma = true;
    } else if (!komma) {
      setzeAnzeige(anzeigeText + ".");
      komma = true;
    }
  }

  private void aktionGrundrechenartTaste(String pOperator){
    recheneinheit.verarbeiteGrundrechenoperator(gibAnzeigeAlsZahl());
    setzeAnzeige("" + recheneinheit.gibZahlspeicher());
    recheneinheit.speichereGrundrechenoperator(pOperator);
    neueZahl = true;
    komma = false;
  }

  private void aktionGleichTaste(){
    recheneinheit.verarbeiteGrundrechenoperator(gibAnzeigeAlsZahl());
    setzeAnzeige("" + recheneinheit.gibZahlspeicher());
    recheneinheit.speichereGrundrechenoperator("");
    neueZahl = true;
    komma = false;
  }

  private void aktionWurzelTaste(){
    double ergebnis =recheneinheit.liefereQuadratwurzel(gibAnzeigeAlsZahl()); 
    setzeAnzeige("" + ergebnis);
    neueZahl = true;
    komma = false;
  }

  private void aktionProzentTaste(){
    double ergebnis = recheneinheit.liefereProzentwert(gibAnzeigeAlsZahl());
    setzeAnzeige("" + ergebnis);
    neueZahl = true;
    komma = false;
  }

  private void aktionVorzeichenTaste(){
    double ergebnis = recheneinheit.liefereGegenzahl(gibAnzeigeAlsZahl());
    setzeAnzeige("" + ergebnis);
    neueZahl = true;
    komma = false;
  }

  private void aktionAllClearTaste(){
    recheneinheit.zuruecksetzen();
    setzeAnzeige("0");
    neueZahl = true;
    komma = false;
  }
  // -------------- ENDE: Ereignismethoden -------------- 

  public static void main(String[] args) {
    GLKamera kamera = new GLKamera(300, 475);     
    GLLicht licht = new GLLicht();

    TaschenrechnerGUI rechner = new TaschenrechnerGUI();
    rechner.fuehreAus();
  }
}